@@ -25,20 +25,12 @@ Pydoc-Markdown to implement custom loaders for documentation data,
 processors or renderers.
 """
 
-from .util.databind import ChainTypeResolver
-from nr.databind.core import  SerializeAs, UnionType
-from nr.databind.core.union import EntrypointTypeResolver, ImportTypeResolver
-from nr.interface import Interface, default
-from typing import Iterable, List, Optional
-import docspec
+import abc
 import subprocess
+import typing as t
 
-
-def _make_union_type(entrypoint_name: str) -> UnionType:
-  return UnionType(ChainTypeResolver(
-    EntrypointTypeResolver(entrypoint_name),
-    ImportTypeResolver(),
-  ))
+import docspec
+from databind.core import annotations as A
 
 
 class Context:
@@ -50,14 +42,14 @@ class Context:
     self.directory = directory
 
 
-class PluginBase(Interface):
+class PluginBase(abc.ABC):
 
-  @default
-  def init(self, context: Context) -> None:
-    pass
+  def init(self, context: Context) -> None: pass
 
 
-@SerializeAs(_make_union_type('pydoc_markdown.interfaces.Loader'))
+@A.unionclass(A.unionclass.Subtypes.chain(
+  A.unionclass.Subtypes.entrypoint('pydoc_markdown.interfaces.Loader'),
+  A.unionclass.Subtypes.import_()), style=A.unionclass.Style.flat)
 class Loader(PluginBase):
   """
   This interface describes an object that is capable of loading documentation
@@ -65,26 +57,26 @@ class Loader(PluginBase):
   with the configuration class.
   """
 
-  def load(self) -> Iterable[docspec.Module]:
-    """
-    Load modules.
-    """
+  @abc.abstractmethod
+  def load(self) -> t.Iterable[docspec.Module]: ...
 
 
 class LoaderError(Exception):
   pass
 
 
-class Resolver(Interface):
+class Resolver(abc.ABC):
   """
   A resolver can be used by a #Processor to replace cross references with a hyperlink.
   """
 
-  def resolve_ref(self, scope: docspec.ApiObject, ref: str) -> Optional[str]:
-    pass
+  @abc.abstractmethod
+  def resolve_ref(self, scope: docspec.ApiObject, ref: str) -> t.Optional[str]: ...
 
 
-@SerializeAs(_make_union_type('pydoc_markdown.interfaces.Processor'))
+@A.unionclass(A.unionclass.Subtypes.chain(
+  A.unionclass.Subtypes.entrypoint('pydoc_markdown.interfaces.Processor'),
+  A.unionclass.Subtypes.import_()), style=A.unionclass.Style.flat)
 class Processor(PluginBase):
   """
   A processor is an object that takes a list of #docspec.Module#s as an input and
@@ -92,11 +84,13 @@ class Processor(PluginBase):
   various documentation syntaxes to plain Markdown.
   """
 
-  def process(self, modules: List[docspec.Module], resolver: Optional[Resolver]) -> None:
-    pass
+  @abc.abstractmethod
+  def process(self, modules: t.List[docspec.Module], resolver: t.Optional[Resolver]) -> None: ...
 
 
-@SerializeAs(_make_union_type('pydoc_markdown.interfaces.Renderer'))
+@A.unionclass(A.unionclass.Subtypes.chain(
+  A.unionclass.Subtypes.entrypoint('pydoc_markdown.interfaces.Renderer'),
+  A.unionclass.Subtypes.import_()), style=A.unionclass.Style.flat)
 class Renderer(PluginBase):
   """
   A renderer is an object that takes a list of #docspec.Module#s as an input and produces
@@ -108,19 +102,15 @@ class Renderer(PluginBase):
   used without the actual renderering functionality, #Renderer is a subclass of #Processor.
   """
 
-  @default
-  def process(self, modules: List[docspec.Module], resolver: Optional[Resolver]) -> None:
-    pass
+  def process(self, modules: t.List[docspec.Module], resolver: t.Optional[Resolver]) -> None: pass
 
-  @default
-  def get_resolver(self, modules: List[docspec.Module]) -> Optional[Resolver]:
-    return None
+  def get_resolver(self, modules: t.List[docspec.Module]) -> t.Optional[Resolver]: return None
 
-  def render(self, modules: List[docspec.Module]) -> None:
-    pass
+  @abc.abstractmethod
+  def render(self, modules: t.List[docspec.Module]) -> None: ...
 
 
-class Server(Interface):
+class Server(abc.ABC):
   """
   This interface describes an object that can start a server process for
   live-viewing generated documentation in the browser. #Renderer
@@ -129,13 +119,12 @@ class Server(Interface):
   CLI.
   """
 
-  def get_server_url(self) -> str:
-    ...
+  @abc.abstractmethod
+  def get_server_url(self) -> str: ...
 
-  def start_server(self) -> subprocess.Popen:
-    ...
+  @abc.abstractmethod
+  def start_server(self) -> subprocess.Popen: ...
 
-  @default
   def reload_server(self, process: subprocess.Popen) -> subprocess.Popen:
     """
     Called when the files generated by pydoc-markdown have been updated.
@@ -147,27 +136,29 @@ class Server(Interface):
     return process
 
 
-class Builder(Interface):
+class Builder(abc.ABC):
   """
   This interface can be implemented additionally to the #Renderer interface to
   indicate that the renderer supports building another produce after the markdown
   files have been rendered.
   """
 
-  def build(self, site_dir: str=None) -> None:
+  @abc.abstractmethod
+  def build(self, site_dir: str) -> None:
     """
-    Invoke the build. If *site_dir* is specified, it is the directory in which the
-    output files should be placed. Otherwise, the directory may be determined by
-    the builder.
+    Invoke the build. The *site_dir* is the directory in which the output files should be
+    placed. Otherwise, the directory may be determined by the builder.
     """
 
 
-@SerializeAs(_make_union_type('pydoc_markdown.interfaces.SourceLinker'))
+@A.unionclass(A.unionclass.Subtypes.chain(
+  A.unionclass.Subtypes.entrypoint('pydoc_markdown.interfaces.SourceLinker'),
+  A.unionclass.Subtypes.import_()), style=A.unionclass.Style.flat)
 class SourceLinker(PluginBase):
   """
   This interface is used to determine the URL to the source of an API object. Renderers
   can use it to place a link to the source in the generated documentation.
   """
 
-  def get_source_url(self, obj: docspec.ApiObject) -> Optional[str]:
-    pass
+  @abc.abstractmethod
+  def get_source_url(self, obj: docspec.ApiObject) -> t.Optional[str]: ...
